var AppConstants = require('../constants/AppConstants');
var AppDispatcher = require('../dispatchers/AppDispatcher');
var EventEmitter = require('events').EventEmitter;
var Session = require('./Session');
var assign = require('object-assign');

var MY_PROJECTS_LOADED = 'my_projects_loaded';
var ALL_PROJECTS_LOADED = 'all_projects_loaded';
var PROJECT_CREATED = 'project_created';
var PROJECT_CREATION_FAILED = 'project_creation_failed';
var MEMBER_ADDED = 'member_added';
var MEMBER_ADD_FAILED = 'member_add_failed';

var cache = {
  'my': null,
  'all': null
};

var Projects = assign({}, EventEmitter.prototype, {

  getMyProjects: function () {
    return cache['my'];
  },

  getAllProjects: function () {
    return cache['all'];
  },

  invalidateCache: function () {
    cache['my'] = null;
    cache['all'] = null;
  },

  listMyProjects: function () {
    this.invalidateCache();

    var client = Session.getClient();
    var ajax = client.listMySyncProjects();

    var self = this;
    ajax.done(function (data) {
      cache['my'] = data;
      self.emitMyProjectsLoaded();
    });
  },

  listAllProjects: function () {
    this.invalidateCache();

    var client = Session.getClient();
    var ajax = client.listAllSyncProjects();

    var self = this;
    ajax.done(function (data) {
      cache['all'] = data;
      self.emitAllProjectsLoaded();
    });
  },

  createProject: function (name, description, depotDir) {
    this.invalidateCache();

    var client = Session.getClient();

    var opts = {name: name};
    if (description) {
      opts.description = description;
    }
    if (depotDir) {
      opts.depotDir = depotDir;
    }

    var ajax = client.createSyncProject(opts);

    var self = this;

    ajax.done(function (project) {
      self.emitProjectCreated(project);
    });

    ajax.fail(function () {
      self.emitProjectCreationFailed();
    });
  },

  addMemberToProject: function (project, login) {
    this.invalidateCache();

    var client = Session.getClient();

    project.members.push(login);

    var ajax = client.updateSyncProject(project);

    var self = this;

    ajax.done(function (updated) {
      self.emitMemberAdded(updated, login);
    });

    ajax.fail(function () {
      self.emitMemberAddFailed(project, login);
    });
  },

  emitMyProjectsLoaded: function () {
    this.emit(MY_PROJECTS_LOADED, this.getMyProjects());
  },

  addMyProjectsLoadedListener: function (callback) {
    this.on(MY_PROJECTS_LOADED, callback);
  },

  removeMyProjectsLoadedListener: function (callback) {
    this.removeListener(MY_PROJECTS_LOADED, callback);
  },

  emitAllProjectsLoaded: function () {
    this.emit(ALL_PROJECTS_LOADED, this.getAllProjects());
  },

  addAllProjectsLoadedListener: function (callback) {
    this.on(ALL_PROJECTS_LOADED, callback);
  },

  removeAllProjectsLoadedListener: function (callback) {
    this.removeListener(ALL_PROJECTS_LOADED, callback);
  },

  emitProjectCreated: function (project) {
    this.emit(PROJECT_CREATED, project);
  },

  addProjectCreatedListener: function (callback) {
    this.on(PROJECT_CREATED, callback);
  },

  removeProjectCreatedListener: function (callback) {
    this.removeListener(PROJECT_CREATED, callback);
  },

  emitProjectCreationFailed: function () {
    this.emit(PROJECT_CREATION_FAILED);
  },

  addProjectCreationFailedListener: function (callback) {
    this.on(PROJECT_CREATION_FAILED, callback);
  },

  removeProjectCreationFailedListener: function (callback) {
    this.removeListener(PROJECT_CREATION_FAILED, callback);
  },

  emitMemberAdded: function (project, login) {
    this.emit(MEMBER_ADDED, project, login);
  },

  addMemberAddedListener: function (callback) {
    this.on(MEMBER_ADDED, callback);
  },

  removeMemberAddedListener: function (callback) {
    this.removeListener(MEMBER_ADDED, callback);
  },

  emitMemberAddFailed: function (project, login) {
    this.emit(MEMBER_ADD_FAILED, project, login);
  },

  addMemberAddFailedListener: function (callback) {
    this.on(MEMBER_ADD_FAILED, callback);
  },

  removeMemberAddFailedListener: function (callback) {
    this.removeListener(MEMBER_ADD_FAILED, callback);
  }
});

AppDispatcher.register(function (action) {
  switch (action.actionType) {
    case AppConstants.CREATE_PROJECT:
      Projects.createProject(action.name, action.description, action.depotDir);
      break;

    case AppConstants.LIST_MY_PROJECTS:
      Projects.listMyProjects();
      break;

    case AppConstants.LIST_ALL_PROJECTS:
      Projects.listAllProjects();
      break;

    case AppConstants.ADD_MEMBER:
      Projects.addMemberToProject(action.project, action.login);
      break;

    default:
    // Do nothing
  }
});

module.exports = Projects;
